#!/usr/bin/fades

import os
import shutil
import time
import webbrowser
from collections import Counter
from urllib import request

import clize  # fades
import flickrapi  # fades

APIKEY = open("/home/facundo/.flickr/api_key.txt").read().strip()
SECRET = open("/home/facundo/.flickr/sharedsecret.txt").read().strip()
NSID = "54757453@N00"


def get_photos_from_set(flickr, set_id):
    print("Getting photo list from set", set_id)
    page = 1
    all_photos = []
    while True:
        resp = flickr.photosets.getPhotos(photoset_id=set_id, page=str(page))
        assert resp['stat'] == 'ok'
        photoset = resp['photoset']
        all_photos.extend(photoset['photo'])
        if int(photoset['page']) == photoset['pages']:
            # we're right in the last page
            break
        page += 1
    return all_photos


def get_collection_info(flickr, collection_id):
    print("Getting photo info from collection", collection_id)
    tree = flickr.collections.getTree(collection_id=collection_id)
    assert tree['stat'] == 'ok'
    (collection,) = tree['collections']['collection']
    all_sets = collection['set']

    all_photos = []
    for set_ in all_sets:
        print("Getting photos from set {!r} ({})".format(set_['title'], set_['description']))
        all_photos.extend(get_photos_from_set(flickr, set_['id']))
    return all_photos


def get_tagged_photos(flickr, tag):
    print("Getting photo list from tag", repr(tag))
    page = 1
    all_photos = []
    while True:
        resp = flickr.photos.search(user_id="me", tags=tag, page=str(page))
        assert resp['stat'] == 'ok'
        photos = resp['photos']
        all_photos.extend(photos['photo'])
        if int(photos['page']) == photos['pages']:
            # we're right in the last page
            break
        page += 1
    return all_photos


def download_photo(flickr, photo_id, photo_name):
    resp = flickr.photos.getSizes(photo_id=photo_id)
    for size_data in resp['sizes']['size']:
        if size_data['label'] == 'Original':
            photo_url = size_data['source']
            break
    else:
        print("ERROR!!!!!!! Original size not found for photo id", repr(photo_id))
        return

    # download image to temp file
    http_resp = request.urlopen(photo_url)
    temp_photo_name = photo_name + ".tmp"
    with open(temp_photo_name, "wb") as fh:
        shutil.copyfileobj(http_resp, fh)

    # all fine, rename to proper name
    os.rename(temp_photo_name, photo_name)


FUNCS = {
    'collections': get_collection_info,
    'sets': get_photos_from_set,
    'tags': get_tagged_photos,
}

BASE_PHOTO = "https://www.flickr.com/photos/54757453@N00/"


def main(group_type, group_id, *, show_non_public=False, download=False):
    """Main entry point."""
    retr_func = FUNCS[group_type]

    # auth
    print("Authenticating...")
    flickr = flickrapi.FlickrAPI(APIKEY, SECRET, format='parsed-json')
    if not flickr.token_valid(perms='read'):
        # get a request token
        flickr.get_request_token(oauth_callback='oob')

        # open a browser at the authentication URL. Do this however
        # you want, as long as the user visits that URL.
        authorize_url = flickr.auth_url(perms='read')
        webbrowser.open_new_tab(authorize_url)

        # get the verifier code from the user. Do this however you
        # want, as long as the user gives the application the code.
        verifier = input('Verifier code from the web page just opened: ')

        # trade the request token for an access token
        flickr.get_access_token(verifier)

    all_photos = retr_func(flickr, group_id)
    print("Found {} photos".format(len(all_photos)))

    # summary
    cnt = Counter()
    not_publics = {}
    for photo in all_photos:
        status = []
        for level in ('ispublic', 'isfamily', 'isfriend'):
            if photo[level]:
                status.append(level)
        status = ' '.join(status)
        cnt[status] += 1
        if status != 'ispublic':
            not_publics.setdefault(status, []).append(photo)

    for status, q in cnt.most_common():
        print(" {:5d} {}".format(q, status))
        if show_non_public:
            for photo in not_publics.get(status, []):
                print("        {:12s} {}".format(photo['title'], BASE_PHOTO + photo['id']))

    if download:
        for idx, photo in enumerate(all_photos, 1):
            if os.path.exists(photo['title']):
                print("Skipping {title} (already downloaded) {idx}/{total}".format(
                    **photo, idx=idx, total=len(all_photos)))
                continue

            print("Downloading {title} (id={id}) {idx}/{total}".format(
                **photo, idx=idx, total=len(all_photos)))
            download_photo(flickr, photo['id'], photo['title'])

            # wait a little to be gentle to the server
            time.sleep(1)
    print("Done.")


if __name__ == "__main__":
    # ej: ./get_flickr_data.py collections 1786914-72157628528931731
    clize.run(main)
